/*
 * @(#)BattleShips.java 1.0 03/08/26
 *
 * Written by James McMahon
 */

import java.awt.*;
import java.util.*;
import java.applet.*;
import javax.swing.*;
import javax.swing.border.*;
import java.net.*;

public class Board extends JPanel {

	public final static int small  = 0;//6x6
	public final static int medium = 1;//10x10
	public final static int large  = 2;//15x15

	//helps when determining what location a mouse click refers to in the array
	private int actualBoxesAcross = 0;
	private int actualBoxesDown   = 0;

	//set values for the standard board sizes
	private final static int smallBoxesAcross  = 6;
	private final static int smallBoxesDown    = 6;
	private final static int mediumBoxesAcross = 10;
	private final static int mediumBoxesDown   = 10;
	private final static int largeBoxesAcross  = 15;
	private final static int largeBoxesDown    = 15;

	//The status of the display of the ships - true = display them, false = hide them
	private boolean visibilityStatus = false;

	//the drop box locations list
	private BoxLocationsList listOfLocations = new BoxLocationsList();

	//the numbers that are held in the boardRep array
	public final static int blank         = 0;
	public final static int missed_shot   = 1;
	public final static int hit_shot      = 2;

	//any value above 10 is regarded as a ship square of type
	public final static int ship_battleship = 11;
	public final static int ship_cruiser    = 12;
	public final static int ship_destroyer1 = 13;//first destroyer
	public final static int ship_destroyer2 = 14;//second destroyer
	public final static int ship_mtb1		 = 15;//first mtb boat
	public final static int ship_mtb2		 = 16;//second mtb boat
	public final static int ship_mtb3		 = 17;//third mtb boat

	//the number of boxes each ship type occupies
	private final static int battleship_length = 5;
	private final static int cruiser_length    = 4;
	private final static int destroyer_length  = 3;
	private final static int mtb_length		   = 2;

	//the number of boxes of a ship type that has currently been placed
	private int battleShipsBoxesPlaced     = 0;
	private int cruiserShipsBoxesPlaced	   = 0;
	private int destroyer1ShipsBoxesPlaced = 0;
	private int destroyer2ShipsBoxesPlaced = 0;
	private int mtb1ShipsBoxesPlaced 	   = 0;
	private int mtb2ShipsBoxesPlaced       = 0;
	private int mtb3ShipsBoxesPlaced       = 0;

	//the number of unhit squares each ship has on the board
	private int battleshipStatus = 5;
	private int cruiserStatus    = 4;
	private int destroyer1Status = 3;
	private int destroyer2Status = 3;
	private int mtb1Status       = 2;
	private int mtb2Status       = 2;
	private int mtb3Status       = 2;

	//tracks what a sot does to the ships on the board after a dealWithMouseClickShot
	private boolean lastShotSunkShip = false;
	private boolean lastShotHitShip  = false;
	private String  lastShipSunk = null;//the type of ship subk if any

	//stores the location of the last ship point that was placed on the board
	private BoxLocation lastShipPlacedBox = null;

	//The main parent class
	private BattleShips parent = null;

	/**
	 * sets up a blank battleships board
	 *
	 * @param type  an integer specifying either
	 *			<code>Board.small</code> or
	 *          <code>Board.medium</code> or
	 *			<code>Board.large</code>
	 * @param ship_status The display status of the ships (true = display, false = hide)
	 * @param parentOwner gives access to the change player function
	 */
	public Board(int type, boolean ship_status, BattleShips parentOwner) {
		this.setSize(300,300);
		this.setPreferredSize(new Dimension(300,300));
		visibilityStatus = ship_status;
		parent = parentOwner;

		this.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));

		if (type == 2) {//the large board size
			actualBoxesAcross = largeBoxesAcross;
			actualBoxesDown   = largeBoxesDown;
		}
		else if (type == 1) {//the medium size board
			actualBoxesAcross = mediumBoxesAcross;
			actualBoxesDown   = mediumBoxesDown;
		}
		else {//give the default small board
			actualBoxesAcross = smallBoxesAcross;
			actualBoxesDown   = smallBoxesDown;
		}

		//set everything in the board rep to be a blank box location
		createBlankBoardArray();
	}

	/**
	 * Sets up a blank battleships board
	 *
	 * @param numBoxesAcross The number of boxes across the board is (5 min, 20 max)
	 * @param numBoxesDown The number of boxes down the board is (5 min, 20 max)
	 * @param ship_status The display status of the ships (true = display, false = hide)
	 * @param parentOwner gives access to the change player function
	 */
	public Board(int numBoxesAcross, int numBoxesDown, boolean ship_status, BattleShips parentOwner) {

		this.setSize(300,300);
		this.setPreferredSize(new Dimension(300,300));
		visibilityStatus = ship_status;
		parent = parentOwner;

		this.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));

		if (numBoxesAcross > 20) {
			actualBoxesAcross = 20;//the maximum allowed
		}
		else if (numBoxesAcross < 5) {
			actualBoxesAcross = 5;//the minimum	allowed
		}
		else {
			actualBoxesAcross = numBoxesAcross;
		}

		if (numBoxesDown > 20) {
			actualBoxesDown = 20;//the maximum allowed
		}
		else if (numBoxesDown < 5) {
			actualBoxesDown = 5;//the minimum allowed
		}
		else {
			actualBoxesDown = numBoxesDown;
		}

		//set everything in the board rep to be a blank box location
		createBlankBoardArray();
	}

	/**
	 * Creates and populates the list of the board with blank boxes
	 */
	private void createBlankBoardArray() {
		int xBoxWidth  = (this.getPreferredSize().getSize().width/this.actualBoxesDown);
		int yBoxHeight = (this.getPreferredSize().getSize().height/this.actualBoxesAcross);

		//set everything in the board rep to be a blank box location
		for (int i = 0; i<this.actualBoxesAcross; i++) {
			for (int j = 0; j<this.actualBoxesDown; j++) {

				int x1 = xBoxWidth*j;
				int x2 = (xBoxWidth*j)+xBoxWidth;
				int y1 = yBoxHeight*i;
				int y2 = (yBoxHeight*i)+yBoxHeight;

				Point topLeft     = new Point(x1,y1);
				Point bottomRight = new Point(x2,y2);

				listOfLocations.addBoxLocationToList(new BoxLocation(topLeft, bottomRight ,new Point(i,j), this.blank));
		    }
	    }
	}

	/**
	 * Changes the player to the other person and makes it
	 */
	public void finishedFiring() {
		parent.makeComputerPlayerMove();
	}

	/**
	 * Returns whether or not the ships locations on the board are to be drawn or not.
	 * Computer opponent board should be occluded until it is game over
	 *
	 * @return The status of ships visibility for the board
	 */
	public boolean isShipsLocationsVisible() {
		return visibilityStatus;
	}

	/**
	 * Sets the visibility status of the ships placed on to this board
	 *
	 * @param The status to make the ships display on the board (true for on, false for off)
	 */
	public void setShipsLocationsVisible(boolean visibleStatus) {
		visibilityStatus = visibleStatus;
	}


	/**
	 * Determines whether or not box is logicaly next to the last placed ship location.
	 * @returns Whether it was or not
	 */
	public boolean boxNearLastLocation(BoxLocation box) {
		if (lastShipPlacedBox != null) {
			Point a = box.getReference();
			Point b = lastShipPlacedBox.getReference();

			int newX = a.x;
			int newY = a.y;
			int lastPlacedX = b.x;
			int lastPlacedY = b.y;

			//valid is when the location is directly up, down, left or
			//right of the lastplacedbox square by one place

			if ((((newX+1)==lastPlacedX) && (newY==lastPlacedY)) || //match to the right
				(((newX-1)==lastPlacedX) && (newY==lastPlacedY)) || //match to the left
				((newX==lastPlacedX) && ((newY-1)==lastPlacedY)) || //match to the top
			    ((newX==lastPlacedX) && ((newY+1)==lastPlacedY))) { //match to the bottom
				return true;
			}
		}//else
		return false;
	}

	/**
	 * Sets up the board for the computer player
	 */
	public void setUpShipLocations(BoxLocationsList list) {
		this.listOfLocations = list;
	}

	/**
	 * Resets the last placed ship location and the ship associated with it
	 */
	public void resetCurrentShipPlacement() {
		//first find the last placed box if possible
		if (lastShipPlacedBox != null) {

			String ship = null;
			int locationsToFill = 0;

			int contains = lastShipPlacedBox.getContents();
			if (contains == this.ship_battleship) {
				this.battleShipsBoxesPlaced=0;//sets it back to zero
				ship = "BattleShip";
				locationsToFill = this.battleship_length;
			}
			else if (contains == this.ship_cruiser) {
				this.cruiserShipsBoxesPlaced=0;
				ship = "Cruiser";
				locationsToFill = this.cruiser_length;
			}
			else if (contains == this.ship_destroyer1) {
				this.destroyer1ShipsBoxesPlaced=0;
				ship = "1st Destroyer";
				locationsToFill = this.destroyer_length;
			}
			else if (contains == this.ship_destroyer2) {
				this.destroyer2ShipsBoxesPlaced=0;
				ship = "2nd Destroyer";
				locationsToFill = this.destroyer_length;
			}
			else if (contains == this.ship_mtb1) {
				this.mtb1ShipsBoxesPlaced=0;
				ship = "1st Motor Torpedo Boat";
				locationsToFill = this.mtb_length;
			}
			else if (contains == this.ship_mtb2) {
				this.mtb2ShipsBoxesPlaced=0;
				ship = "2nd Motor Torpedo Boat";
				locationsToFill = this.mtb_length;
			}
			else if (contains == this.ship_mtb3) {
				this.mtb3ShipsBoxesPlaced=0;
				ship = "3rd Motor Torpedo Boat";
				locationsToFill = this.mtb_length;
			}

			clearAllShips(contains);
			lastShipPlacedBox = null;
			this.validate();
			this.repaint();

			JOptionPane.showMessageDialog(this,
										  new JLabel("You may now replace your "+ship+" ("+locationsToFill+" locations)"),
								  		  "Placing Ships",
								   	 	  JOptionPane.INFORMATION_MESSAGE);
		}
	}

	/**
	 * Clears all of the ships on the board relating to type shipType
	 */
	private void clearAllShips(int shipType) {
		//set everything in the board rep to be a blank box location if it matches shipType

		ListIterator i = listOfLocations.getListOfAllBoxes().listIterator();

		while (i.hasNext()) {
			BoxLocation box = (BoxLocation)i.next();
			int boxContents = box.getContents();
			if (boxContents == shipType) {
				box.setContents(Board.blank);
			}
		}
	}

	/**
	 * Deals with a mouse click that was to place a ship
	 *
	 * @param location The location of the mouse click
	 * @return Whether the placement of a ship point was sucessful or not
	 */
	public boolean dealWithShipPlacementMouseClick(Point location) {
		BoxLocation box = listOfLocations.getBoxLocationContainingPoint(location);
		if (box != null) {
			return processShipPlacement(box);
		}
		else {
			return false;
		}
	}

	/**
	 * Primarily intended for use by the AI player. This function randomly places
	 * a ship shipType onto the board
	 *
	 * @param type Must bo one of Board.ship_battleship etc
	 */
	public void placeRandomShip(int shipType) {
		int shipLength = 0;
		int shipsCounter = 0;

		//find out how many squares need to be placed
		if (shipType == Board.ship_battleship) {
			shipLength = Board.battleship_length;
		}
		else if (shipType == Board.ship_cruiser) {
			shipLength = Board.cruiser_length;
		}
		else if ((shipType == Board.ship_destroyer1) ||
				 (shipType == Board.ship_destroyer2)) {
			shipLength = Board.destroyer_length;
		}
		else if ((shipType == Board.ship_mtb1) ||
				 (shipType == Board.ship_mtb2) ||
				 (shipType == Board.ship_mtb3)) {
			shipLength = Board.mtb_length;
		}

		ArrayList validBlanks = this.listOfLocations.getListOfBlankBoxes();
		Random rand = new Random();
		int ranLoc;

		//place the ships squares
		for (int i = 0; i < shipLength; i++) {
			//put in a square to a blank box
			ranLoc = rand.nextInt(validBlanks.size());
			BoxLocation box = (BoxLocation)validBlanks.get(ranLoc);

			box.setContents(shipType);
			shipsCounter++;
	   		lastShipPlacedBox = box;

	   		//find out if more squares are available if they are needed
			validBlanks = this.listOfLocations.getSurroundingBlankBoxes(box);
	    	if ((validBlanks.isEmpty()) && (i < shipLength) && (shipsCounter!=shipLength)) {
	    		//then we can't finsh placing and need to start again

	    		System.out.println("can't complete i = "+i+", shipsCounter = "+shipsCounter+", shipLength="+shipLength);

	    		this.clearAllShips(shipType);//first clear the ship squares
	    		i = -1;						 //then reset the counters
				shipsCounter = 0;//resetting i to -1 seems to fix a weird problem - bit of a hack at the moment
	   			lastShipPlacedBox = null;
	   			validBlanks = this.listOfLocations.getListOfBlankBoxes();
	    	}
	    }

		//update the number of boxes placed in order to keep things all straight
		if (shipType == Board.ship_battleship) {
			battleShipsBoxesPlaced = shipsCounter;
		}
		else if (shipType == Board.ship_cruiser) {
			cruiserShipsBoxesPlaced = shipsCounter;
		}
		else if (shipType == Board.ship_destroyer1) {
			destroyer1ShipsBoxesPlaced = shipsCounter;
		}
		else if (shipType == Board.ship_destroyer2) {
			destroyer2ShipsBoxesPlaced = shipsCounter;
		}
		else if (shipType == Board.ship_mtb1) {
			mtb1ShipsBoxesPlaced = shipsCounter;
		}
		else if (shipType == Board.ship_mtb2) {
			mtb2ShipsBoxesPlaced = shipsCounter;
		}
		else if (shipType == Board.ship_mtb3) {
			mtb3ShipsBoxesPlaced = shipsCounter;
		}
	}

	/**
	 * Primarily intended for use by the AI player. This function randomly places
	 * a ship shipType onto the board but tries to space them out so none touch (if space allows)
	 * *Based on the surrounding squares of each new ship square) - sometimes makes a difference
	 * and sometimes doesn't - perfect for confusing opponents - well kinda.
	 *
	 * @param type Must bo one of Board.ship_battleship etc
	 */
	public void placeSpacedRandomShip(int shipType) {
		int shipLength = 0;
		int shipsCounter = 0;
		int numOfTimesTried = 0;

		//find out how many squares need to be placed
		if (shipType == Board.ship_battleship) {
			shipLength = Board.battleship_length;
		}
		else if (shipType == Board.ship_cruiser) {
			shipLength = Board.cruiser_length;
		}
		else if ((shipType == Board.ship_destroyer1) ||
				 (shipType == Board.ship_destroyer2)) {
			shipLength = Board.destroyer_length;
		}
		else if ((shipType == Board.ship_mtb1) ||
				 (shipType == Board.ship_mtb2) ||
				 (shipType == Board.ship_mtb3)) {
			shipLength = Board.mtb_length;
		}

		ArrayList validBlanks = this.listOfLocations.getListOfBlankBoxes();
		Random rand = new Random();
		int ranLoc;

		//place the ships squares
		for (int i = 0; i < shipLength; i++) {
			//put in a square to a blank box
			ranLoc = rand.nextInt(validBlanks.size());
			BoxLocation box = (BoxLocation)validBlanks.get(ranLoc);

			box.setContents(shipType);
			shipsCounter++;
			lastShipPlacedBox = box;

			//find out if more squares are available if they are needed
			validBlanks = this.listOfLocations.getSurroundingBlankBoxes(box);

			if (((validBlanks.isEmpty()) && (i < shipLength) && (shipsCounter!=shipLength)) ||
				((validBlanks.size() <= 3) && (numOfTimesTried <= 3))) {

				numOfTimesTried++;

				//then we can't finsh placing and need to start again
				this.clearAllShips(shipType);//first clear the ship squares
				i = -1;						 //then reset the counters
				shipsCounter = 0;//resetting i to -1 seems to fix a weird problem - bit of a hack at the moment
				lastShipPlacedBox = null;
				validBlanks = this.listOfLocations.getListOfBlankBoxes();
			}
		}

		//update the number of boxes placed in order to keep things all straight
		if (shipType == Board.ship_battleship) {
			battleShipsBoxesPlaced = shipsCounter;
		}
		else if (shipType == Board.ship_cruiser) {
			cruiserShipsBoxesPlaced = shipsCounter;
		}
		else if (shipType == Board.ship_destroyer1) {
			destroyer1ShipsBoxesPlaced = shipsCounter;
		}
		else if (shipType == Board.ship_destroyer2) {
			destroyer2ShipsBoxesPlaced = shipsCounter;
		}
		else if (shipType == Board.ship_mtb1) {
			mtb1ShipsBoxesPlaced = shipsCounter;
		}
		else if (shipType == Board.ship_mtb2) {
			mtb2ShipsBoxesPlaced = shipsCounter;
		}
		else if (shipType == Board.ship_mtb3) {
			mtb3ShipsBoxesPlaced = shipsCounter;
		}
	}

	/**
	 * Deals with a manually specified box for ship placement on the board.
	 *
	 * @param xAcross The across reference location from 0 to actualBoxesAcross
	 * @param yDown The down reference location from 0 to actualBoxesDown
	 *
	 * @return Whether the ship placement was valid or not
	 */
	public boolean dealWithManualShipPlacement(int xAcross, int yDown) {
		BoxLocation box = listOfLocations.getBoxLocation(new Point(xAcross, yDown));
		if (box != null) {
			return processShipPlacement(box);
		}
		else {
			return false;
		}
	}

	/**
	 * Deals with the effects of a ship being placed onto box
	 * @param box The box that has been targeted for the next ship location
	 * @return Whether the ship placement was valid or not
	 */
	private boolean processShipPlacement(BoxLocation box) {
		if (box != null) {//got a location on the board
			if (box.getContents() == Board.blank) {//then still a valid ship location

				//this is set up so that all battleships get placed first
				//then cruiser, then destroyer1 and 2 and then the 3 mtbs

				if (battleShipsBoxesPlaced < Board.battleship_length) {
					//still placing a battleship
					if ((battleShipsBoxesPlaced>0) && boxNearLastLocation(box)) {
						//the location is valid and we can place the ship
						box.setContents(Board.ship_battleship);
						lastShipPlacedBox = box;
						battleShipsBoxesPlaced++;
						if (battleShipsBoxesPlaced==Board.battleship_length) {
							this.validate();
							this.repaint();
							JOptionPane.showMessageDialog(this,
								new JLabel("Now place your Cruiser (4 locations)"),
								  		   "Placing Ships",
								   	 	   JOptionPane.INFORMATION_MESSAGE);
						}
						return true;
					}
					else if (battleShipsBoxesPlaced==0) {//first box can be anywhere
						box.setContents(Board.ship_battleship);
						lastShipPlacedBox = box;
						battleShipsBoxesPlaced++;
						return true;
					}
				}
				else if (cruiserShipsBoxesPlaced < Board.cruiser_length) {
					//still placing a cruiser
					if ((cruiserShipsBoxesPlaced>0) && boxNearLastLocation(box)) {
						box.setContents(Board.ship_cruiser);
						lastShipPlacedBox = box;
						cruiserShipsBoxesPlaced++;
						if (cruiserShipsBoxesPlaced==Board.cruiser_length) {
							this.validate();
							this.repaint();
							JOptionPane.showMessageDialog(this,
														  new JLabel("Now place your 1st Destroyer (3 locations)"),
														  "Placing Ships",
														  JOptionPane.INFORMATION_MESSAGE);
						}
						return true;
					}
					else if (cruiserShipsBoxesPlaced==0) {//first box can be anywhere
						box.setContents(Board.ship_cruiser);
						lastShipPlacedBox = box;
						cruiserShipsBoxesPlaced++;
						return true;
					}
				}
				else if (destroyer1ShipsBoxesPlaced < Board.destroyer_length) {
					//still placing the first destroyer
					if ((destroyer1ShipsBoxesPlaced>0) && boxNearLastLocation(box)) {
						box.setContents(Board.ship_destroyer1);
						lastShipPlacedBox = box;
						destroyer1ShipsBoxesPlaced++;
						if (destroyer1ShipsBoxesPlaced==Board.destroyer_length) {
							this.validate();
							this.repaint();
							JOptionPane.showMessageDialog(this,
														  new JLabel("Now place your 2nd Destroyer (3 locations)"),
														  "Placing Ships",
														  JOptionPane.INFORMATION_MESSAGE);
						}
						return true;
					}
					else if (destroyer1ShipsBoxesPlaced==0) {//first box can be anywhere
						box.setContents(Board.ship_destroyer1);
						lastShipPlacedBox = box;
						destroyer1ShipsBoxesPlaced++;
						return true;
					}
				}
				else if (destroyer2ShipsBoxesPlaced < Board.destroyer_length) {
					//still placing the second destroyer
					if ((destroyer2ShipsBoxesPlaced>0) && boxNearLastLocation(box)) {
						box.setContents(Board.ship_destroyer2);
						lastShipPlacedBox = box;
						destroyer2ShipsBoxesPlaced++;
						if (destroyer2ShipsBoxesPlaced==Board.destroyer_length) {
							this.validate();
							this.repaint();
 							JOptionPane.showMessageDialog(this,
														  new JLabel("Place your 1st Motor Torpedo Boat (2 locations)"),
														  "Placing Ships",
														  JOptionPane.INFORMATION_MESSAGE);
						}
						return true;
					}
					else if (destroyer2ShipsBoxesPlaced==0) {//first box can be anywhere
						box.setContents(Board.ship_destroyer2);
						lastShipPlacedBox = box;
						destroyer2ShipsBoxesPlaced++;
						return true;
					}
				}
				else if (mtb1ShipsBoxesPlaced < Board.mtb_length) {
					//still placing the first mtb
					if ((mtb1ShipsBoxesPlaced>0) && boxNearLastLocation(box)) {
						box.setContents(Board.ship_mtb1);
						lastShipPlacedBox = box;
						mtb1ShipsBoxesPlaced++;
						if (mtb1ShipsBoxesPlaced==Board.mtb_length) {
							this.validate();
							this.repaint();
 							JOptionPane.showMessageDialog(this,
														  new JLabel("Place your 2nd Motor Torpedo Boat (2 locations)"),
														  "Placing Ships",
														  JOptionPane.INFORMATION_MESSAGE);
						}
						return true;
					}
					else if (mtb1ShipsBoxesPlaced==0) {//first box can be anywhere
						box.setContents(Board.ship_mtb1);
						lastShipPlacedBox = box;
						mtb1ShipsBoxesPlaced++;
						return true;
					}
				}
				else if (mtb2ShipsBoxesPlaced < Board.mtb_length) {
					//still placing the second mtb
					if ((mtb2ShipsBoxesPlaced>0) && boxNearLastLocation(box)) {
						box.setContents(Board.ship_mtb2);
						lastShipPlacedBox = box;
						mtb2ShipsBoxesPlaced++;
						if (mtb2ShipsBoxesPlaced==Board.mtb_length) {
							this.validate();
							this.repaint();
							JOptionPane.showMessageDialog(this,
														  new JLabel("Place your 3rd Motor Torpedo Boat (2 locations)"),
														  "Placing Ships",
														  JOptionPane.INFORMATION_MESSAGE);
						}
						return true;
					}
					else if (mtb2ShipsBoxesPlaced==0) {//first box can be anywhere
						box.setContents(Board.ship_mtb2);
						lastShipPlacedBox = box;
						mtb2ShipsBoxesPlaced++;
						return true;
					}
				}
				else if (mtb3ShipsBoxesPlaced < Board.mtb_length) {
					//still placing the third mtb
					if ((mtb3ShipsBoxesPlaced>0) && boxNearLastLocation(box)) {
						box.setContents(Board.ship_mtb3);
						lastShipPlacedBox = box;
						mtb3ShipsBoxesPlaced++;
						if (mtb3ShipsBoxesPlaced==Board.mtb_length) {
							this.validate();
							this.repaint();

							//as all ships placed we should give this info to the main applet
							parent.allShipsPlaced();//essentially disables the clear ship menu

							JOptionPane.showMessageDialog(this,
														  new JLabel("All ships placed! Fire at your opponent!"),
														  "Placing Ships Finished",
														  JOptionPane.INFORMATION_MESSAGE);
						}
						return true;
					}
					else if (mtb3ShipsBoxesPlaced==0) {//first box can be anywhere
						box.setContents(Board.ship_mtb3);
						lastShipPlacedBox = box;
						mtb3ShipsBoxesPlaced++;
						return true;
					}
				}
			}
			else {//then a ship location or something else is already here and a ship can't be placed
				  //or the point wasn't logically next to the previous ship location
				return false;
			}
		}//click point was outside the board ranges
		return false;
	}

	/**
	 * Returns whether or not all ship square have been properly placed or not
	 */
	public boolean allShipsPlaced() {

		return ((battleShipsBoxesPlaced     == Board.battleship_length)	&&
				(cruiserShipsBoxesPlaced    == Board.cruiser_length)	&&
				(destroyer1ShipsBoxesPlaced == Board.destroyer_length)	&&
				(destroyer2ShipsBoxesPlaced == Board.destroyer_length)	&&
				(mtb1ShipsBoxesPlaced       == Board.mtb_length)		&&
				(mtb2ShipsBoxesPlaced       == Board.mtb_length)		&&
				(mtb3ShipsBoxesPlaced       == Board.mtb_length));
	}

	/**
	 * Returns whether or not all ships have been sunk or not
	 */
	public boolean allShipsSunk() {

		return ((this.battleshipStatus <= 0) &&
				(this.cruiserStatus    <= 0) &&
				(this.destroyer1Status <= 0) &&
				(this.destroyer2Status <= 0) &&
				(this.mtb1Status       <= 0) &&
				(this.mtb2Status	   <= 0) &&
				(this.mtb3Status 	   <= 0));
	}

	/**
	 * Deals with a manually specified box to be shot at on the board.
	 *
	 * @param xAcross The across reference location from 0 to actualBoxesAcross
	 * @param yDown The down reference location from 0 to actualBoxesDown
	 *
	 * @return Whether the shot was valid or not
	 */
	public boolean dealWithManualShot(int xAcross, int yDown) {
		BoxLocation box = listOfLocations.getBoxLocation(new Point(xAcross, yDown));
		if (box != null) {
			return processShot(box);
		}
		else {
			return false;
		}
	}

	/**
	 * Deals with a mouse click that was to fire a shot
	 *
	 * @param location The location of the mouse click
	 * @return Whether the shot was valid or not
	 */
	public boolean dealWithShotMouseClick(Point location) {
		BoxLocation box = listOfLocations.getBoxLocationContainingPoint(location);
		if (box != null) {
			return processShot(box);
		}
		else {
			return false;
		}
	}

	/**
	 * Returns whether or not the last shot hit a ship
	 */
	public boolean lastShotHitShip() {
		return lastShotHitShip;
	}

	/**
	 * Returns whether or not the last shot sunk a ship
	 */
	public boolean lastShotSunkShip() {
		return lastShotSunkShip;
	}

	/**
	 * Deals with the effects of a shot fired into targetedBox
	 * @param targetedBox The box that has been fired into
	 * @return Whether the shot was valid or not
	 */
	private boolean processShot(BoxLocation targetedBox) {
		lastShotSunkShip = false;
		lastShipSunk = null;
		lastShotHitShip = false;

		if (targetedBox != null) {
			if (targetedBox.getContents() >= Board.ship_battleship) {//then a ship location has been hit

				lastShotHitShip = true;

				if (targetedBox.getContents() == Board.ship_battleship) {
					this.battleshipStatus--;
				}
				else if (targetedBox.getContents() == Board.ship_cruiser) {
					this.cruiserStatus--;
				}
				else if (targetedBox.getContents() == Board.ship_destroyer1) {
					this.destroyer1Status--;
				}
				else if (targetedBox.getContents() == Board.ship_destroyer2) {
					this.destroyer2Status--;
				}
				else if (targetedBox.getContents() == Board.ship_mtb1) {
					this.mtb1Status--;
				}
				else if (targetedBox.getContents() == Board.ship_mtb2) {
					this.mtb2Status--;
				}
				else if (targetedBox.getContents() == Board.ship_mtb3) {
					this.mtb3Status--;
				}

				//now reset the contents
				targetedBox.setContents(Board.hit_shot);

				//make a note of the fact of whether or not the last shot sunk a ship
				if (this.battleshipStatus == 0) {
					lastShotSunkShip = true;
					lastShipSunk = "Battleship";
					this.battleshipStatus--;//then register it as totally dead and buried
				}
				else if (this.cruiserStatus == 0) {
					lastShotSunkShip = true;
					lastShipSunk = "Cruiser";
					this.cruiserStatus--;//then register it as totally dead and buried
				}
				else if (this.destroyer1Status == 0) {
					lastShotSunkShip = true;
					lastShipSunk = "Destroyer";
					this.destroyer1Status--;//then register it as totally dead and buried
				}
				else if (this.destroyer2Status == 0) {
					lastShotSunkShip = true;
					lastShipSunk = "Destroyer";
					this.destroyer2Status--;//then register it as totally dead and buried
				}
				else if (this.mtb1Status == 0) {
					lastShotSunkShip = true;
					lastShipSunk = "MTB";
					this.mtb1Status--;//then register it as totally dead and buried
				}
				else if (this.mtb2Status == 0) {
					lastShotSunkShip = true;
					lastShipSunk = "MTB";
					this.mtb2Status--;//then register it as totally dead and buried
				}
				else if (this.mtb3Status == 0) {
					lastShotSunkShip = true;
					lastShipSunk = "MTB";
					this.mtb3Status--;//then register it as totally dead and buried
					//this extra bit stops the same dead ship being continually reported
				}
				else {
					lastShotSunkShip = false;
					lastShipSunk = null;
				}

				if (lastShotSunkShip) {
					if (!this.allShipsSunk()) {
						JOptionPane.showMessageDialog(this,
													  new JLabel(lastShipSunk+" Sunk!"),
													  "Ship Sunk",
													  JOptionPane.INFORMATION_MESSAGE);
					}
					else {
						JOptionPane.showMessageDialog(this,
													  new JLabel("All Ships Sunk!"),
													  "Game Over",
													  JOptionPane.INFORMATION_MESSAGE);
					}
				}
				this.validate();
				this.repaint();
				return true;
			}
			else if (targetedBox.getContents() == Board.blank){//they missed the ships
				targetedBox.setContents(Board.missed_shot);
				lastShotSunkShip = false;
				lastShotHitShip = false;
				lastShipSunk = null;
				this.validate();
				this.repaint();
				return true;
			}
			else {//the must have hit either a hit_shot or missed_shot location so don't
			      //let them fire at it again
			    lastShotSunkShip = false;
				lastShipSunk = null;
				this.validate();
				this.repaint();
			    return false;
			}
		}//the box doesn't actually exist (outside the board ranges in the first place)
		this.validate();
		this.repaint();
		return false;
	}

	/**
	 * Given input "SMALL", "MEDIUM" or "LARGE" it returns the number of boxes that it has
	 * across. Otherwise returns 6.
	 */
	public static int numberOfBoxesAcross(String type) {
		if (type.equals("SMALL")) {//the small board
			return smallBoxesAcross;
		}
		else if (type.equals("MEDIUM")) {//the medium size board
			return mediumBoxesAcross;
		}
		else if (type.equals("LARGE")) {//the large board size
			return largeBoxesAcross;
		}
		else {
			return 6;
		}
	}

	/**
	 * Given input "SMALL", "MEDIUM" or "LARGE" it returns the number of boxes that it has
	 * down. Otherwise returns 6.
	 */
	public static int numberOfBoxesDown(String type) {
		if (type.equals("SMALL")) {//the small board
			return smallBoxesDown;
		}
		else if (type.equals("MEDIUM")) {//the medium size board
			return mediumBoxesDown;
		}
		else if (type.equals("LARGE")) {//the large board size
			return largeBoxesDown;
		}
		else {
			return 6;
		}
	}

	/**
	 * Returns the locations that can still be shot at
	 */
	public BoxLocationsList getValidShotLocations() {
		return new BoxLocationsList(this.listOfLocations.getListOfTargetableBoxes());
	}

	//This is stubbed at the moment because it simply allows the pretty pictures to be
	//randomly allocated
	//eventually it should animate an explosion but will probably require a more complex
	//grid contents system (mainly because I was cheeky and didn't use separate classes
	//to hold the contents of each box - oh well!
	Random rand = new Random(); //STUBBED

	/**
	 * Paints the board based upon the current state.
	 * i.e. it will have either the ships painted on
	 *      or/and the shots that have been fired at it
	 * depending on whether it is supposed to be a user or computer board
	 */
	public void paintComponent(Graphics g) {


		super.paintComponent(g);
		g.setColor(Color.black);

		ListIterator i = listOfLocations.getListOfAllBoxes().listIterator();

		while (i.hasNext()) {
			BoxLocation box = (BoxLocation)i.next();

			int boxContents   = box.getContents();
			Point topLeft     = box.getTopLeftBoundaryPoint();
			Point bottomRight = box.getBottomRightBoundaryPoint();

			int x1 = topLeft.x;
			int y1 = topLeft.y;
			int width = bottomRight.x-topLeft.x;
			int height = bottomRight.y-topLeft.y;

			if (boxContents == this.blank) {
				g.setColor(Color.black);
				g.drawRect(x1,y1,width,height);
			}
			else if (boxContents == this.hit_shot ) {
				//STUBBED draw hit shot tile
				g.setColor(Color.red);
				g.drawRect(x1,y1,width,height);
				g.drawString("H",x1+(width/2)-3,y1+(height/2)+5);

				//STUBBED should eventually be an animation of an explosion
				//images are sized 32 by 32 with a 1 pixel border around each of the sides

				try {//to put up a temporary explosion gif
					ImageIcon im = new ImageIcon(new URL(parent.getCodeBase(), "explosions.gif"));
					Image ima = im.getImage();
					ima.getScaledInstance(width,height, Image.SCALE_SMOOTH);

					int numImages = rand.nextInt(4);

					int sx1 = 0+(34*numImages);
					int sy1 = 0;
					int sx2 = sx1+34;
					int sy2 = 34;

					Graphics2D g2 = (Graphics2D)g;

					g2.drawImage(ima, x1, y1, x1+width, y1+height,
						 			  sx1, sy1, sx2, sy2, this);
				}
				catch (Exception lazy) {
					lazy.printStackTrace();
					//part of the temporary explosion showing piece
				}
				//STILL STUBBED
			}
			else if (boxContents == this.missed_shot) {
				//STUBBED draw missed shot tile
				g.setColor(Color.blue);
				g.drawRect(x1,y1,width,height);
				g.drawString("M",x1+(width/2)-3,y1+(height/2)+5);
			}
			else if ((boxContents == this.ship_battleship) && (isShipsLocationsVisible())) {
				//STUBBED draw battleship ship location tile
				g.setColor(Color.yellow);
				g.drawRect(x1,y1,width,height);
				g.drawString("B",x1+(width/2)-3,y1+(height/2)+5);
			}
			else if ((boxContents == this.ship_cruiser) && (isShipsLocationsVisible())) {
				//STUBBED draw cruiser ship location tile
				g.setColor(Color.yellow);
				g.drawRect(x1,y1,width,height);
				g.drawString("C",x1+(width/2)-3,y1+(height/2)+5);
			}
			else if (((boxContents == this.ship_destroyer1) ||
					 (boxContents == this.ship_destroyer2)) && (isShipsLocationsVisible())) {
				//STUBBED draw a destroyer ship location tile
				g.setColor(Color.yellow);
				g.drawRect(x1,y1,width,height);
				g.drawString("D",x1+(width/2)-3,y1+(height/2)+5);
			}
			else if (((boxContents == this.ship_mtb1) ||
					  (boxContents == this.ship_mtb2) ||
					  (boxContents == this.ship_mtb3)) && (isShipsLocationsVisible())) {
				//STUBBED draw a motor boat ship location tile
				g.setColor(Color.yellow);
				g.drawRect(x1,y1,width,height);
				g.drawString("T",x1+(width/2)-3,y1+(height/2)+5);
			}
			else {
				//STUBBED draw a blank tile - will be necessary anytime we encounter a problem
				//or the ship display status is false
				g.setColor(Color.black);
				g.drawRect(x1,y1,width,height);
			}
		}
	}
}