/*
 * @(#)BattleShips.java 1.0 03/08/26
 *
 * Written by James McMahon
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.applet.*;
import javax.swing.*;
import javax.swing.border.*;

public class BoardMouseListener implements MouseListener {
	
	private Board parent = null;
	
	private boolean firingStatus;
	private boolean stopListening = false;
	
	/**
	 * Sets up the mouse listener to listen for user clicks on the board
	 * 
	 * @param parentBoard The board that the listener is set to
	 * @param firing If true then when the user clicks it is treated as a shot otherwise 
	 *               as ship placement 
	 */
	public BoardMouseListener(Board parentBoard, boolean firing) {
		parent = parentBoard;
		firingStatus = firing;
		stopListening = false;
	}
	
	/**
	 * Sets the listener to deal with shots
	 */
	public void setFiring() {
		firingStatus = true;
	}
	
	/**
	 * sets the listener to deal with ship laying clicks
	 */
	public void setShipLaying() {
		firingStatus = false;
	}
	
	/**
	 * Stops the listener reacting to shots and ship laying
	 */
	public void turnOffListener() {
		stopListening = true;	
	}

	/**
	 * Starts the listener reacting to shots and ship laying
	 */
	public void turnOnListener() {
		stopListening = false;	
	}	
	
    /**
     * Invoked when the mouse button has been clicked (pressed
     * and released) on a component.
     */
    public void mouseClicked(MouseEvent e) {//do nothing
    }

    /**
     * Invoked when a mouse button has been pressed on a component.
     */
    public void mousePressed(MouseEvent e) {
		if(!stopListening) {
	    	if (firingStatus) { 
    			boolean legalShot = parent.dealWithShotMouseClick(e.getPoint());
    			if (legalShot) {
					parent.finishedFiring();
    			} 
    			else {
    				JOptionPane.showMessageDialog(parent, 
									 		  "You can't shoot into this square. Try again!",
									 		  "Non-Valid Shot",
									          JOptionPane.ERROR_MESSAGE);
   		 		}
    		}
    		else {
    			boolean status = parent.dealWithShipPlacementMouseClick(e.getPoint());
   		 		if (!status && !parent.allShipsPlaced()) {//attempt failed and not because all ships placed
   		 			JPanel panel = new JPanel(new BorderLayout());
    		
    				panel.add(new JLabel("Can't place the ship there!"), BorderLayout.NORTH);
   	 				panel.add(new JPanel(), BorderLayout.CENTER);
    				panel.add(new JLabel("<HTML>Ships must be one to the left, right, top <br> or bottom of the last placed square</HTML>"), BorderLayout.SOUTH);
    			
    				JOptionPane.showMessageDialog(parent, 
									 		  panel,
									 		  "Non-Valid Ship Square",
									          JOptionPane.ERROR_MESSAGE);
    			}
    			if (parent.allShipsPlaced()) {
    				this.setFiring();	
    			}
    		}
    	}
    	parent.validate();
    	parent.repaint();
    }

    /**
     * Invoked when a mouse button has been released on a component.
     */
    public void mouseReleased(MouseEvent e) {//do nothing
    }

    /**
     * Invoked when the mouse enters a component.
     */
    public void mouseEntered(MouseEvent e) {//do nothing
    }

    /**
     * Invoked when the mouse exits a component.
     */
    public void mouseExited(MouseEvent e) {//do nothing
    }
}