import java.awt.*;

/*
 * @(#)BattleShips.java 1.0 03/08/26
 *
 * Written by James McMahon
 */
 
public class BoxLocation {
	
	private Point a = new Point(0,0);//top left co-ordinate
	private Point b = new Point(0,0);//bottom right co-ordinate
	private Point reference = new Point(-1,-1);//the reference into the array of the box this represents
	private int contents;//The contents of this box
	
	/**
	 * Creates a DropBox with points topLeft and bottomRight as defining points.
	 * 
	 * @param topLeft The first defining point
	 * @param bottomRight The second defining point
	 * @param arrayReference The point that identifies the box inside an array of the board locations
	 * @param boxContains The internal contents of the box (either Board.missed_shot, Board.blank, Board.hit_shot, or one of the Board ship types)
	 */
	public BoxLocation(Point topLeft, Point bottomRight, Point arrayReference, int boxContains) {
		a     = topLeft;
		b     = bottomRight;
		reference = arrayReference;
		contents  = boxContains;
	}
	
	/**
	 * Get a point that helps mark the box location
	 *
	 * @return The top left point that partially defines the boundary position of this box
	 */
	public Point getTopLeftBoundaryPoint() {
		return a;
	}
	
	/**
	 * Get a point that helps mark the box location
	 *
	 * @return The bottom right point that partially defines the boundary position of this box
	 */
	public Point getBottomRightBoundaryPoint() {
		return b;	
	}
	
	/**
	 * Checks to see if point target is contained within the sphere of this boxes
	 * boundaries
	 * 
	 * @param target The point that is to be checked
	 *
	 * @return Whether or not the target is in this box
	 */
	public boolean containsPoint(Point target) {
		
		int x1 = (int)this.a.getX();
		int y1 = (int)this.a.getY();
		
		int x2 = (int)this.b.getX();
		int y2 = (int)this.b.getY();
		
		int xT = (int)target.getX();
		int yT = (int)target.getY();
		
		if ((x1 <= xT) && (xT <= x2)) {//in the x range so far so good
			if ((y1 <= yT) && (yT <= y2)) {//in the right y range so point matches 
				return true;
			}
		}
		
		//else return false
		return false;
	}
	
	/**
	 * returns the name/location of the box location that this relates to
	 * 
	 * @return The point that has the x,y reference ints into an array of these locations
	 */
	public Point getReference() {
		return reference;	
	}
	
	/**
	 * Sets the representation of what this box actually holds
	 * 
	 * @param contentType Should be either Board.blank, Board.hit_shot, Board.missed_shot, or one of the Board ship types
	 */
	public void setContents(int contentType) {
		contents = contentType;
	}
	
	/**
	 * Returns the representation of what this box actually holds
	 * 
	 * @return Board.blank, Board.missed_shot, Board.hit_shot, or one of the Board ship types depending on contents
	 */
	public int getContents() {
		return contents;	
	}
	
	/**
	 * A string representation of this object
	 */
	public String toString() {
		return "[Box location at {topright :"+a.toString()+" , bottomleft :"+b.toString()+"} with array reference point - "+reference.toString();
	}
}