/*
 * @(#)BattleShips.java 1.0 03/08/26
 *
 * Written by James McMahon
 */

import java.awt.*;
import java.applet.*;
import javax.swing.*;

public class MenuBarPanel extends JMenuBar {
	
	private JMenu settingsMenu = new JMenu("Settings");
	private JMenu aboutMenu    = new JMenu("About");
	
	private JMenuItem newGame          = new JMenuItem("New Game");
	private JMenuItem modSettings      = new JMenuItem("Modify Settings");
	private JMenuItem clearCurrentShip = new JMenuItem("Clear current ship");
	
	private JMenuItem help = new JMenuItem("Help");
	private JMenuItem about = new JMenuItem("About ...");
	
	/**
	 * The listener to deal with events caused by the user clicking on the menu items
	 */
	private MenuBarPanelListener listener;
	
	/**
	 * Creates a standard menubar that can be added to the JApplet
	 * @param parentOwner The main parent class to allow the listener access to necessary functions
	 */
	public MenuBarPanel(BattleShips parentOwner) {
		
		//setup and add the listeners to the menu items
		listener = new MenuBarPanelListener(parentOwner);
		
		newGame.addActionListener(listener);
		modSettings.addActionListener(listener);
		clearCurrentShip.addActionListener(listener);
		help.addActionListener(listener);
		about.addActionListener(listener);
			
		//set up the mneumonics
		newGame.setMnemonic('n');
		modSettings.setMnemonic('m');
		clearCurrentShip.setMnemonic('c');
		help.setMnemonic('h');
		about.setMnemonic('a');
		
		//set up the actioncommands
		newGame.setActionCommand("new");
		modSettings.setActionCommand("settings");
		clearCurrentShip.setActionCommand("clearplacedship");
		help.setActionCommand("help");
		about.setActionCommand("about");
		
		//set up the menubar
		settingsMenu.add(newGame);
		settingsMenu.add(modSettings);
		settingsMenu.addSeparator();
		settingsMenu.add(clearCurrentShip);
		
		aboutMenu.add(help);
		aboutMenu.addSeparator();
		aboutMenu.add(about);

		this.add(settingsMenu);
		this.add(aboutMenu);
	}
	
	public void setClearShipEnabled(boolean enabled) {
		clearCurrentShip.setEnabled(enabled);
	}
}