/*
 * @(#)BattleShips.java 1.0 03/08/26
 *
 * Written by James McMahon
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;

public class MenuBarPanelListener implements ActionListener {
	
	/**
	 * The parent owner that allows access to the necessary functions
	 */
	private BattleShips parent = null;
	
	/** 
	 * Creates the listener for menu events
	 * @param parentOwner The main applet class to allow access to the necessary functions 
	 */
	public MenuBarPanelListener(BattleShips parentOwner) {
		parent = parentOwner;
	}
	
	/**
	 * The actionperformed method to deal with any ActionEvents that get fired
	 */
	public void actionPerformed(ActionEvent ae) {
		String action = ae.getActionCommand();
		
		if (action.equals("new")) {
			parent.startNewGame();
		}
		else if (action.equals("settings")) {
			OptionsPanel optionsPanel = new OptionsPanel();
			
			int returnValue = JOptionPane.showConfirmDialog(parent.getContentPane(), 
															optionsPanel,
						 				 					"BattleShips Options",
						        		 					JOptionPane.OK_CANCEL_OPTION,
						        							JOptionPane.PLAIN_MESSAGE,
						        		  					null);
			if (returnValue == 0) {
				String boardSize = optionsPanel.getBoardSizeWanted();
				int computerPlayerLevel = optionsPanel.getComputerPlayerLevel();
				int boxesAcross;
				int boxesDown;
				
				if (boardSize.equals("CUSTOM")) { 
					if (optionsPanel.isCustomBoardValuesSet()) {
						//want to pass on and get the custom values	
						boxesAcross = optionsPanel.getBoxesAcrossCustomSettings();
						boxesDown   = optionsPanel.getBoxesDownCustomSettings();
					}
					else {
						//this occurs if the user clicked CUSTOM board size but didn't set the board
						//values - this is a hacked out version at the moment - I'll
						//ix it in the future so that it doesn't require manual setting
						boxesAcross = Board.numberOfBoxesAcross("SMALL");
						boxesDown   = Board.numberOfBoxesDown("SMALL");
					}
				}
				else {
					//want to pass on the standard values of boardsize asked for
					boxesAcross = Board.numberOfBoxesAcross(boardSize);
					boxesDown   = Board.numberOfBoxesDown(boardSize);
				}
				parent.setOptions(boxesAcross, boxesDown, computerPlayerLevel);
				parent.startNewGame();
			}
			//else the user cancelled so don't need to do anything to the options
		}
		else if (action.equals("clearplacedship")) {
			parent.resetCurrentShipPlacement();
		}
		else if (action.equals("help")) {
			new HelpPanel(parent.getCodeBase());
		}
		else if (action.equals("about")) {
			
			double versionNumber = 1.0;
			
			JPanel panel = new JPanel();
			panel.setPreferredSize(new Dimension(100,100));
			panel.setBorder(BorderFactory.createEtchedBorder());
			panel.setLayout(new BorderLayout());
			
			panel.add(new JLabel("<HTML>This is the first incarnation <br> of my battleships game!</HTML>", JLabel.CENTER), BorderLayout.NORTH);
			panel.add(new JLabel("<HTML>Version: "+versionNumber+"</HTML>", JLabel.CENTER), BorderLayout.CENTER);
			panel.add(new JLabel("<HTML>Suggestions, bugs etc - send to <br> <a href=\"mailto:jamesmcmahon@contactme.co.uk\">jamesmcmahon@contactme.co.uk</a></HTML>", JLabel.CENTER), BorderLayout.SOUTH);
			
			JOptionPane.showMessageDialog(parent.getContentPane(), 
								 		  panel,
								 		  "BattleShips Applet Version "+versionNumber,
								          JOptionPane.CLOSED_OPTION);
		}
	}
}